<?php

namespace Drupal\lb_copy_section;

use Drupal\Core\Render\Element;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;
use Drupal\node\NodeInterface;

/**
 * Implements preRender for layout builder.
 */
class CopySectionRender implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRender'];
  }

  /**
   * Alters layout builder to only allow one text block in title regions
   */
  public static function preRender($element) {
    $lb = &$element['layout_builder'];
    $tempstore = \Drupal::service('tempstore.private')->get('lb_copy_section');
    $section_storage = $element['#section_storage'];

    foreach (Element::children($lb) as $section) {

      // Add Paste links
      if (isset($lb[$section]['link'])
        && isset($lb[$section]['link']['#url'])
        && !empty($tempstore->get('copied_section'))
      ) {

        // Collect parameters to generate the link
        $params = $lb[$section]['link']['#url']->getRouteParameters();
        $copied_section = $tempstore->get('copied_section');
        $layout_settings = $copied_section->getLayoutSettings();
        $section_label = !empty($tempstore->get('copied_section_label')) ? $tempstore->get('copied_section_label') : t('Section');

        $paste_section_url = Url::fromRoute(
          'lb_copy_section.paste',
          [
            'section_storage_type' => $params['section_storage_type'],
            'section_storage' => $params['section_storage'],
            'delta' => $params['delta'],
          ],
          [
            'attributes' => [
              'class' => [
                'use-ajax',
                'layout-builder__link',
                'layout-builder__link--add',
                'layout-builder__link--paste-section',
              ],
              'data-dialog-type' => 'dialog',
              'data-dialog-renderer' => 'off_canvas',
            ],
          ]
        );
        $lb[$section]['paste'] = [
          '#type' => 'link',
          '#title' => t('Paste @section', ['@section' => $section_label]),
          '#url' => $paste_section_url,
          '#access' => $paste_section_url->access(),
        ];
      }

      // Add Copy links
      elseif (isset($lb[$section]['configure']) && isset($lb[$section]['configure']['#url'])) {

        // Collect parameters to generate the link
        $params = $lb[$section]['configure']['#url']->getRouteParameters();
        $active_section = $section_storage->getSections()[$params['delta']];
        $layout_settings = $active_section->getLayoutSettings();
        $section_label = !empty($layout_settings['label']) ? $layout_settings['label'] : t('Section @section', ['@section' => $params['delta'] + 1]);

        // Pop the last item to add the link before it.
        $last_item_section = array_pop($lb[$section]);

        $copy_section_url = Url::fromRoute(
          'lb_copy_section.copy',
          [
            'section_storage_type' => $params['section_storage_type'],
            'section_storage' => $params['section_storage'],
            'delta' => $params['delta'],
          ],
          [
            'attributes' => [
              'class' => [
                'use-ajax',
                'layout-builder__link',
                'layout-builder__link--copy-section',
              ],
              'data-dialog-type' => 'dialog',
              'data-dialog-renderer' => 'off_canvas',
            ],
          ]
        );
        $lb[$section]['copy'] = [
          '#type' => 'link',
          '#title' => t('Copy @section', ['@section' => $section_label]),
          '#url' => $copy_section_url,
          '#access' => $copy_section_url->access(),
        ];

        // Push the last item again.
        array_push($lb[$section], $last_item_section);
      }
    }

    // Load library
    if(\Drupal::currentUser()->hasPermission('copy paste section')) {
      $element['#attached']['library'][] = 'lb_copy_section/admin';
    }

    return $element;
  }
}
